/***************************************************************************
 *   Copyright (C) 2006 by Leon Pennington                                 *
 *   leon@leonscape.co.uk                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "applyjob.h"
#include "feeds.h"

#include <qdir.h>
#include <qfile.h>
#include <qtimer.h>

#include <kio/job.h>
#include <kprocess.h>
#include <kmessagebox.h>
#include <ktempfile.h>
#include <klocale.h>
#include <kurlcompletion.h>

ApplyJob::ApplyJob(QObject *parent, Feeds *feeds, const QString &cfgDir, const QString &dataDir, const QString &interval, const QString &log, int subDirType )
 : QObject( parent, 0 )
{
    m_pFeeds = feeds;
    m_configDir = cfgDir;
    m_dataDir = dataDir;
    m_inter = interval;
    m_log = log;
    m_subDirType = subDirType;
    m_pTFile = 0;
}

ApplyJob::~ApplyJob()
{
    delete m_pTFile;
}

void ApplyJob::begin()
{
    QTimer *timer = new QTimer( this );
    connect( timer, SIGNAL( timeout() ), SLOT( slotTimeOut() ) );
    m_timeOut = MKDIR;
    timer->start( 2000, true );

    QDir dir;
    if( dir.exists( m_configDir ) )
        dataDir();
    else
    {
        emit message( i18n( "Creating directory %1" ).arg( m_configDir ) );
        KIO::Job *mkdir = KIO::mkdir( m_configDir );
        connect( mkdir, SIGNAL( result( KIO::Job* ) ), this, SLOT( slotMkConfigDirFinish( KIO::Job* ) ) );
    }
}

void ApplyJob::slotMkConfigDirFinish( KIO::Job *job )
{
    if( job->error() > 0 )
    {
         m_timeOut = INTER;
         job->showErrorDialog( );
         emit failed( i18n( "Creating directory error" ) );
    }
    else
        dataDir();
}

void ApplyJob::slotMkDataDirFinish( KIO::Job *job )
{
    if( job->error() > 0 )
    {
        m_timeOut = INTER;
        job->showErrorDialog( );
        emit failed( i18n( "Creating directory error" ) );
    }
    else
        writeFiles();
}

void ApplyJob::slotCronData( KProcess*, char *buffer, int bufLen )
{
    m_cronData += QString::fromLatin1( buffer, bufLen );
}

void ApplyJob::slotCronError( KProcess*, char *buffer, int bufLen )
{
    m_cronErrStr += QString::fromLatin1( buffer, bufLen );
    m_cronErr = true;
}

void ApplyJob::slotCronDataFinish( KProcess* )
{
    m_timeOut = INTER;

    if( m_cronErr && m_inter.isEmpty() && m_cronData.isEmpty() )
    {
        // Can't check error string since it maybe translated, assuming there was a "no crontab for user" error
        emit success();
        return;
    }

    emit message( i18n( "Replacing crontab" ) );
    QStringList list = QStringList::split( '\n', m_cronData );
    for( unsigned i = 0; i < list.size(); ++i )
    {
        if ( list[i].endsWith( "bashpodder.shell" ) )
        {
            QString other = list[i].section( ' ', -1 );
            if( KMessageBox::questionYesNo( 0, i18n( "Found another bashpodder cron job.\nDo you wish to keep the job %1 ?" ).arg( other ),
                    i18n( "bashpodder cronjob" ) ) == KMessageBox::No )
                list[i] = QString::null;
        }
        else if( list[i].find( QString("kpodder"), 0, false  ) >= 0 )
            list[i] = QString::null;
    }

    KTempFile *m_pTFile = new KTempFile();
    if( m_pTFile->status() != 0 )
    {
        KMessageBox::error( 0, i18n( "Couldn't create a temp file to store crontab." ), i18n( "Temporary file error" ) );
        emit failed( i18n( "Couldn't create a temp file to store crontab." ) );
        return;
    }

    QTextStream *stream = m_pTFile->textStream();
    *stream << "#KPodder entries" << endl;
    if( !m_inter.isEmpty() )
    {
        *stream << m_inter << "kpodder.sh -c \"" << m_configDir << "\" -s \"global.casts\" -o \"" << m_dataDir << "\" -d " << m_subDirType << endl;
    }

    QStringList crons = m_pFeeds->cronFeeds();
    for( QStringList::Iterator it = crons.begin(); it != crons.end(); ++it )
    {
        if( !(*it).isEmpty() )
            *stream << *it << " -c \"" << m_configDir << "\" -o \"" << m_dataDir << "\" -d " << m_subDirType << endl;
    }
    *stream << "#KPodder End" << endl << endl;

    for( unsigned i = 0; i < list.size(); ++i )
    {
        if( !list[i].isNull() )
            *stream << list[i] << endl;
    }
    m_pTFile->close();

    m_timeOut = CRONREPLACE;
    m_cronErr = false;
    m_cronData = m_cronErrStr = QString::null;
    KProcess *proc = new KProcess;
    *proc << "crontab" << m_pTFile->name();
    connect( proc, SIGNAL( receivedStderr( KProcess*, char*, int ) ), SLOT( slotCronError( KProcess*, char*, int ) ) );
    connect( proc, SIGNAL( processExited( KProcess* ) ), SLOT( slotCronReplaceFinish( KProcess* ) ) );
    proc->start( KProcess::NotifyOnExit, KProcess::Stderr );
}

void ApplyJob::slotCronReplaceFinish( KProcess* )
{
    m_timeOut = INTER;
    if( m_cronErr )
    {
        KMessageBox::error( 0, i18n( "Couldn't replace the crontab file.\nError given:\n%1" ).arg( m_cronErrStr ), i18n( "Crontab replace error" ) );
        emit failed( i18n( "Replacing crontab failed" ) );
    }
    else
        emit success();
}

void ApplyJob::slotTimeOut()
{
    switch( m_timeOut )
    {
        case INTER:
            {
                QTimer *timer = new QTimer( this );
                connect( timer, SIGNAL( timeout() ), SLOT( slotTimeOut() ) );
                timer->start( 2000, true );
            }
            break;
        case MKDIR:
            KMessageBox::error( 0, i18n( "Could not make directory %1. Operation timed out." ).arg( m_dataDir ), i18n( "Data directory error" ) );
            emit failed( i18n( "Couldn't create data directory" ) );
            break;
        case CRONLIST:
            if( !m_inter.isEmpty() )
            {
                KMessageBox::error( 0, i18n( "Could not get crontab listing.\nMaybe Crontab is not installed?" ), i18n( "Crontab error" ) );
                emit failed( i18n( "Could not get crontab listing." ) );
            }
            else
                emit success();
            break;
        case CRONREPLACE:
            if( !m_inter.isEmpty() )
            {
                KMessageBox::error( 0, i18n( "Could not replace crontab.\nMaybe theres something wrong with crontab?" ), i18n( "Crontab error" ) );
                emit failed( i18n( "Could not replace crontab." ) );
            }
            else
                emit success();
            break;
    }
}

void ApplyJob::dataDir()
{
    m_timeOut = MKDIR;
    QDir dir;
    if( dir.exists( m_dataDir ) )
        writeFiles();
    else
    {
        emit message( i18n( "Creating directory %1" ).arg( m_dataDir ) );
        KIO::Job *mkdir = KIO::mkdir( m_dataDir );
        connect( mkdir, SIGNAL( result( KIO::Job* ) ), this, SLOT( slotMkDataDirFinish( KIO::Job* ) ) );
    }
}

void ApplyJob::writeFiles()
{
    m_timeOut = INTER;
    emit message( i18n( "Creating file %1general.casts" ).arg( m_configDir ) );
    if ( !m_pFeeds->saveFeeds( m_configDir ) )
    {
        emit failed( i18n( "Couldn't save the feeds file" ) );
        return;
    }

    if( !m_log.isEmpty() )
    {
        QString fileName = m_configDir + "/podcast.log";
        emit message( i18n( "Saving file %1" ).arg( fileName ) );
        QFile file( fileName );
        if( file.open( IO_WriteOnly ) )
        {
            QTextStream stream( &file );
            stream << m_log;
            file.close();
        }
        else
        {
            KMessageBox::error( 0, i18n( "Couldn't save the podcast.log file." ), i18n( "Couldn't save podcast.log" ) );
            emit failed( i18n( "Couldn't save the podcast.log file." ) );
            return;
        }
    }

    emit message( i18n( "Checking crontab list" ) );
    m_cronErr = false;
    m_cronData = m_cronErrStr = QString::null;
    KProcess *proc = new KProcess;
    *proc << "crontab" << "-l";
    connect( proc, SIGNAL( receivedStdout( KProcess*, char*, int ) ), SLOT( slotCronData( KProcess*, char*, int ) ) );
    connect( proc, SIGNAL( receivedStderr( KProcess*, char*, int ) ), SLOT( slotCronError( KProcess*, char*, int ) ) );
    connect( proc, SIGNAL( processExited( KProcess* ) ), SLOT( slotCronDataFinish( KProcess* ) ) );
    m_timeOut = CRONLIST;
    proc->start( KProcess::NotifyOnExit, KProcess::AllOutput );
}

#include "applyjob.moc"

