/***************************************************************************
 *   Copyright (C) 2006 by Leon Pennington                                 *
 *   leon@leonscape.co.uk                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "feeditem.h"
#include "castitem.h"
#include "kpodder.h"

#include <qfile.h>
#include <qdom.h>

#include <kio/netaccess.h>
#include <klocale.h>

FeedItem::FeedItem(QListView *parent, const QString &name, const QString &url, bool global, const QString &interval )
    : KListViewItem( parent, name, url, global ? "Yes" : "No", interval )
{
   m_DLurl = QString::null;
}

void FeedItem::setName( const QString &name )
{
   setText( 0, name );
}

void FeedItem::getNameFromURL( const QString &defName, const QString &url )
{
   download( url );
   if( !m_DLName.isEmpty() )
      setText( 0, m_DLName );
   else if ( !defName.isEmpty() )
      setText( 0, defName );
}

void FeedItem::setURL( const QString &url )
{
   setText( 1, url );
}

void FeedItem::setGlobalInterval( bool global )
{
    if( global )
    {
        setText( 2, "Yes" );
        setText( 3, "" );
    }
    else
        setText( 2, "No" );
}

void FeedItem::setInterval( const QString &interval )
{
    if( !globalInterval() )
        setText( 3, interval );
}

void FeedItem::download( const QString &url )
{
   QString nURL;
   if( url.isEmpty() )
   {
      if( text( 1 ) == m_DLurl )
         return;
      else
         m_DLurl = text( 1 );
   }
   else if ( url == m_DLurl )
      return;
   else
      m_DLurl = url;

   QString tmpFileName;
   m_DLName = QString::null;
   if( KIO::NetAccess::download( m_DLurl, tmpFileName, listView()->topLevelWidget() ) )
   {
      QFile file( tmpFileName );
      if( file.open( IO_ReadOnly ) )
      {
         processFile( &file );

         if( m_DLName.isEmpty() )
         {
            QTextStream stream( &file );
            while( !stream.atEnd() )
            {
               QString line = stream.readLine().simplifyWhiteSpace();
               int start = line.find( "<title>", 0, false );
               if( start > -1 )
               {
                  int end = line.find( "</title>", start, false );
                  file.close();
                  KIO::NetAccess::removeTempFile( tmpFileName );
                  if( end > -1 )
                     m_DLName = line.mid( start + 7, end - ( start + 7 ) );
                  else
                     m_DLName = line.mid( start + 7 );
                  return;
               }
            }
         }
         file.close();
      }
      KIO::NetAccess::removeTempFile( tmpFileName );
   }
}

bool FeedItem::isDownloaded( const QString &url )
{
   if( url.isEmpty() )
   {
      if( text( 1 ) == m_DLurl )
         return true;
      else
         return false;
   }
   else if ( url == m_DLurl )
      return true;
   else
      return false;
}

void FeedItem::casts( KListView *view, const QString &url )
{
   download( url );
   view->clear();

   for( QValueVector<Cast>::iterator Itr = m_casts.begin(); Itr != m_casts.end(); ++Itr )
      new CastItem( view, Itr->name, Itr->url, Itr->file, Itr->dled );
}

void FeedItem::setCasts( KListView *view )
{
   CastItem *item = dynamic_cast<CastItem*>( view->firstChild() );
   KPodder *top = dynamic_cast<KPodder*>( listView()->topLevelWidget() );
   while( item )
   {
      for( QValueVector<Cast>::iterator Itr = m_casts.begin(); Itr != m_casts.end(); ++Itr )
      {
         if( Itr->url == item->url() )
         {
            Itr->dled = item->downloaded();
            break;
         }
      }

      if( item->downloaded() )
         top->addToLog( item->url() );
      else
         top->remFromLog( item->url() );

      item = dynamic_cast<CastItem*>( item->nextSibling() );
   }
}

void FeedItem::processFile( QFile *file )
{
   m_casts.clear();
   QDomDocument doc( "Doc" );
   if( doc.setContent( file ) )
   {
      QDomNode channel, item, node;
      QDomText text;
      QDomElement e;
      channel = doc.namedItem( "rss" ).namedItem( "channel" );
      if( !channel.isNull() )
      {
         node = channel.namedItem( "title" ).firstChild();
         if( !node.isNull() && node.isText() )
         {
            QDomText text = node.toText();
            m_DLName = text.data();
         }

         Cast cast;
         int numCast = 0;
         item = channel.namedItem( "item" );
         while( !item.isNull() )
         {
            node = item.namedItem( "title" ).firstChild();
            if( !node.isNull() && node.isText() )
            {
               text = node.toText();
               cast.name = text.data();
            }
            else
               cast.name = i18n( "Cast %1" ).arg( numCast );

            node = item.namedItem( "enclosure" );
            if( !node.isNull() && node.isElement() )
            {
               e = node.toElement();
               if( e.hasAttribute( "url" ) )
               {
                  cast.url = e.attribute( "url" );
                  cast.file = cast.url.section( "/", -1 );
                  m_casts.append( cast );
               }
            }
            numCast++;
            item = item.nextSibling();
            while( !item.isNull() && ( item.nodeName() != "item" ) )
               item = item.nextSibling();
         }

         KPodder *top = dynamic_cast<KPodder*>( listView()->topLevelWidget() );
         for( QValueVector<Cast>::iterator Itr = m_casts.begin(); Itr != m_casts.end(); ++Itr )
         {
            if( top->inLog( Itr->url ) )
               Itr->dled = true;
            else
               Itr->dled = false;
         }
      }
   }
}
