/***************************************************************************
*   Copyright (C) 2006 by Leon Pennington                                 *
*   leon@leonscape.co.uk                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include "kpodder.h"
#include "applyjob.h"
#include "feedcheck.h"
#include "feeds.h"
#include "interval.h"

#include <qlayout.h>
#include <qlabel.h>

#include <klineedit.h>
#include <kpushbutton.h>
#include <kiconloader.h>
#include <kurlcompletion.h>
#include <kaction.h>
#include <kstdaction.h>
#include <kconfig.h>
#include <kapplication.h>
#include <kfiledialog.h>
#include <kdirselectdialog.h>
#include <kmessagebox.h>
#include <kedittoolbar.h>
#include <klocale.h>
#include <kstatusbar.h>
#include <kcombobox.h>
#include <kstandarddirs.h>
#include <kdebug.h>

KPodder::KPodder() : KMainWindow( 0, "KPodder" )
{
    m_cfgDir = instance()->dirs()->saveLocation( "appdata" );
    setupWindow();
    setupActions();
    setStandardToolBarMenuEnabled( true );
    createStandardStatusBarAction();
    setAutoSaveSettings();
    createGUI();
    loadOptions( instance() ->config() );
    m_logRead = false;
}

void KPodder::addFeed( const QString &url )
{
    m_pFeeds->addFeed( url );
}

bool KPodder::inLog( const QString &url )
{
    readLog();
    for ( QStringList::Iterator itr = m_log.begin(); itr != m_log.end(); ++itr )
    {
        if ( url == *itr )
            return true;
    }
    return false;
}

void KPodder::addToLog( const QString &url )
{
    readLog();
    for ( QStringList::Iterator itr = m_log.begin(); itr != m_log.end(); ++itr )
    {
        if ( url == *itr )
            return ;
    }

    m_log += url;
}

void KPodder::remFromLog( const QString &url )
{
    readLog();
    for ( QStringList::Iterator itr = m_log.begin(); itr != m_log.end(); ++itr )
    {
        if ( url == *itr )
            itr = m_log.erase( itr );
    }
}

void KPodder::saveProperties( KConfig *config )
{
    saveOptions( config );
}

void KPodder::readProperties( KConfig *config )
{
    loadOptions( config );
}

bool KPodder::queryClose()
{
    if ( m_modified )
    {
        int ret = KMessageBox::warningYesNoCancel( 0, i18n( "The settings have been modified.\nDo you want to save your changes or discard them?" ),
                  i18n( "Unsaved Changes" ), i18n( "&Save" ), i18n( "&Discard" ) );
        switch ( ret )
        {
            case KMessageBox::Yes:
                slotApply();
                break;
            case KMessageBox::Cancel:
                return false;
                break;
        }
    }

    if ( !m_pCheckNow->isEnabled() )
        KMessageBox::information( 0, i18n( "bashpodder is still gathering files from the feeds.\nIt will continue when you quit" ), i18n( "bashpodder" ) );

    return true;
}

void KPodder::fileNew()
{
    m_pDataDir->setText( KURLCompletion::replacedPath( "~/podcasts", true ) );
    m_pFeeds->clear();
    m_pInterval->setAsDefault();
    setModified( false );
    m_logRead = false;
    m_log.clear();
}

void KPodder::importConfig()
{
    QString fileName = KFileDialog::getOpenFileName( QString::null, "*.conf|" + i18n( "BashPodder Configuration" ) );
    if ( !fileName.isEmpty() )
    {
        m_pFeeds->loadFeeds( fileName );
        setModified( true );
    }
}

void KPodder::exportConfig()
{
    QString fileName = KFileDialog::getSaveFileName( QString::null, "*.conf|" + i18n( "BashPodder Configuration" ) );
    if ( !fileName.isEmpty() )
    {
        if ( QFile::exists( fileName ) )
        {
            if ( KMessageBox::warningContinueCancel( 0,
                    i18n ( "A file called \"%1\" already exists.\nDo you want to overwrite it?" ).arg( fileName ),
                    QString::null, i18n ( "Overwrite" ) ) == KMessageBox::Cancel )
                return ;
        }

        m_pFeeds->saveFeeds( fileName, true );
    }
}

void KPodder::importLog()
{
    QString fileName = KFileDialog::getOpenFileName( QString::null, "*.log|" + i18n( "BashPodder Log" ) );
    if ( !fileName.isEmpty() )
    {
        QFile file( fileName );
        if ( file.open( IO_ReadOnly ) )
        {
            QStringList inLog;
            QTextStream stream( &file );
            while ( !stream.atEnd() )
                inLog += stream.readLine().simplifyWhiteSpace();

            file.close();

            for( QStringList::iterator inItr = inLog.begin(); inItr != inLog.end(); ++inItr )
                addToLog( *inItr );

            setModified( true );
        }
        else
            KMessageBox::sorry( this, i18n( "Could not open %1 for reading" ).arg( fileName ), i18n( "BashPodder Log" ) );
    }
}

void KPodder::optionsConfigureToolbars()
{
    saveMainWindowSettings( KGlobal::config(), autoSaveGroup() );
    KEditToolbar dlg( factory() );
    connect( &dlg, SIGNAL( newToolbarConfig() ), this, SLOT( applyNewToolbarConfig() ) );
    dlg.exec();
}

void KPodder::applyNewToolbarConfig()
{
    applyMainWindowSettings( KGlobal::config(), autoSaveGroup() );
}

void KPodder::slotDataDirChanged()
{
    setModified( true );
    m_logRead = false;
    m_log.clear();
}

void KPodder::slotDataDir()
{
    KURL newDir = KDirSelectDialog::selectDirectory( m_pDataDir->text(), false, this, i18n( "Select Data Directory" ) );
    if ( !newDir.isEmpty() )
    {
        m_pDataDir->setText( newDir.path( + 1 ) );
        setModified( true );
        m_logRead = false;
        m_log.clear();
    }
}

void KPodder::slotSubDirTypeChanged()
{
    setModified( true );
}

void KPodder::slotCheckNow()
{
    m_pFeedCheck = new FeedCheck( this, KURLCompletion::replacedPath( m_pDataDir->text(), true ), m_cfgDir,
                                  m_pFeeds, m_pSubDirType->currentItem() );
    connect( m_pFeedCheck, SIGNAL( finished( bool ) ), SLOT( slotCheckNowFinish( bool ) ) );
    connect( m_pFeedCheck, SIGNAL( message( const QString& ) ), statusBar(), SLOT( message( const QString& ) ) );
    m_pFeedCheck->begin();
}

void KPodder::slotCheckNowFinish( bool success )
{
    if ( success )
        statusBar() ->message( i18n( "Finished checking podcasts" ), 2000 );
    else
        statusBar() ->message( i18n( "There was an error while checking the feeds" ), 4000 );
    m_pCheckNow->setEnabled( true );
    delete m_pFeedCheck;
}

void KPodder::slotApply()
{
    m_pMain->setEnabled( false );
    QString path = KURLCompletion::replacedPath( m_pDataDir->text(), true );

    m_pApplyJob = new ApplyJob( this, m_pFeeds, m_cfgDir, path, m_pInterval->cronString(),
                                m_log.isEmpty() ? QString::null : m_log.join( "\n" ), m_pSubDirType->currentItem() );
    connect( m_pApplyJob, SIGNAL( success() ), SLOT( slotApplySuccess() ) );
    connect( m_pApplyJob, SIGNAL( failed( const QString& ) ), SLOT( slotApplyFailed( const QString& ) ) );
    connect( m_pApplyJob, SIGNAL( message( const QString& ) ), statusBar(), SLOT( message( const QString& ) ) );
    m_pApplyJob->begin();
}

void KPodder::slotApplySuccess()
{
    m_pMain->setEnabled( true );
    statusBar() ->message( i18n( "Successfully applied changes" ), 2000 );
    setModified( false );
    saveOptions( instance() ->config() );
    delete m_pApplyJob;
}

void KPodder::slotApplyFailed( const QString &error )
{
    m_pMain->setEnabled( true );
    statusBar() ->message( error, 4000 );
    delete m_pApplyJob;
}

void KPodder::slotReset()
{
    loadOptions( instance() ->config() );
}

void KPodder::setupWindow()
{
    m_pMain = new QWidget( this );
    setCentralWidget( m_pMain );

    m_pDirComplete = new KURLCompletion( KURLCompletion::DirCompletion );

    QVBoxLayout *top = new QVBoxLayout( m_pMain, KDialog::marginHint(), KDialog::spacingHint() );

    QGridLayout *gl = new QGridLayout( top, 2, 3 );
    gl->addWidget( new QLabel( i18n( "Data directory:" ), m_pMain ), 0, 0, Qt::AlignRight );
    m_pDataDir = new KLineEdit( m_pMain );
    m_pDataDir->setURLDropsEnabled( true );
    m_pDataDir->setCompletionObject( m_pDirComplete, true );
    gl->addWidget( m_pDataDir, 0, 1 );
    m_pDataDirButton = new KPushButton( SmallIcon( "fileopen" ), QString::null, m_pMain );
    gl->addWidget( m_pDataDirButton, 0, 2 );
    gl->addWidget( new QLabel( i18n( "Sub directory type:" ), m_pMain ), 1, 0, Qt::AlignRight );
    QHBoxLayout *hl = new QHBoxLayout();
    m_pSubDirType = new KComboBox( false, m_pMain );
    QStringList items;
    items << i18n( "Date" ) << i18n( "Feed Name" ) << i18n( "Date/Feed Name" ) << i18n( "Feed Name/Date" );
    m_pSubDirType->insertStringList( items );
    hl->addWidget( m_pSubDirType );
    hl->addStretch();
    gl->addLayout( hl, 1, 1 );

    m_pFeeds = new Feeds( m_pMain );
    top->addWidget( m_pFeeds );

    m_pInterval = new Interval( 0, m_pMain );
    top->addWidget( m_pInterval );

    hl = new QHBoxLayout( top );
    m_pCheckNow = new KPushButton( i18n( "Check Now" ), m_pMain );
    hl->addWidget( m_pCheckNow );
    hl->addStretch();
    m_pApply = new KPushButton( i18n( "Apply" ), m_pMain );
    m_pApply->setEnabled( false );
    hl->addWidget( m_pApply );
    m_pReset = new KPushButton( i18n( "Reset" ), m_pMain );
    m_pReset->setEnabled( false );
    hl->addWidget( m_pReset );

    connect( m_pDataDir, SIGNAL( textChanged( const QString& ) ), SLOT( slotDataDirChanged() ) );
    connect( m_pDataDirButton, SIGNAL( clicked() ), SLOT( slotDataDir() ) );
    connect( m_pSubDirType, SIGNAL( activated( int ) ), SLOT( slotSubDirTypeChanged() ) );
    connect( m_pFeeds, SIGNAL( dataChanged() ), SLOT( slotDataChanged() ) );
    connect( m_pInterval, SIGNAL( dataChanged() ), SLOT( slotDataChanged() ) );
    connect( m_pCheckNow, SIGNAL( clicked() ), SLOT( slotCheckNow() ) );
    connect( m_pApply, SIGNAL( clicked() ), SLOT( slotApply() ) );
    connect( m_pReset, SIGNAL( clicked() ), SLOT( slotReset() ) );
}

void KPodder::setupActions()
{
    KStdAction::openNew( this, SLOT( fileNew() ), actionCollection() );
    KStdAction::quit( this, SLOT( close() ), actionCollection() );
    new KAction( i18n( "Import BashPodder configuration..." ), "fileopen", 0, this, SLOT( importConfig() ), actionCollection(), "import_config" );
    new KAction( i18n( "Export BashPodder configuration..." ), "filesaveas", 0, this, SLOT( exportConfig() ), actionCollection(), "export_config" );
    new KAction( i18n( "Import BashPodder log..." ), "text_left", 0, this, SLOT( importLog() ), actionCollection(), "import_log" );

    // settings menu
    KStdAction::configureToolbars( this, SLOT( optionsConfigureToolbars() ), actionCollection() );
}

void KPodder::setModified( bool modified )
{
    m_modified = modified;
    m_pApply->setEnabled( modified );
    m_pReset->setEnabled( modified );
}

void KPodder::loadOptions( KConfig *config )
{
    config->setGroup( "options" );


    m_pDataDir->setText( config->readEntry( "data_dir", KURLCompletion::replacedPath( "~/podcasts", true ) ) );
    m_pSubDirType->setCurrentItem( config->readNumEntry( "subdir_type", 0 ) );
    m_pInterval->loadInterval( config );
    m_pFeeds->loadFeeds( config );
    m_modified = config->readBoolEntry( "modified", false );
    setModified( m_modified );
}

void KPodder::saveOptions( KConfig *config )
{
    config->deleteGroup( "options" );
    config->setGroup( "options" );
    config->writeEntry( "data_dir", m_pDataDir->text() );
    config->writeEntry( "subdir_type", QString::number( m_pSubDirType->currentItem() ) );
    m_pInterval->saveInterval( config );
    m_pFeeds->saveFeeds( config );
    config->writeEntry( "modified", m_modified );
}

void KPodder::readLog()
{
    if ( m_logRead )
        return ;

    QString fileName = m_cfgDir + "podcast.log";
    QFile file( fileName );
    if ( file.open( IO_ReadOnly ) )
    {
        QTextStream stream( &file );
        while ( !stream.atEnd() )
            m_log += stream.readLine().simplifyWhiteSpace();

        file.close();
    }

    m_logRead = true;
}

#include "kpodder.moc"
